@@ -31,12 +31,15 @@ import gridfs
 from pymatgen.apps.borg.hive import AbstractDrone
 from pymatgen.analysis.structure_analyzer import VoronoiCoordFinder
 from pymatgen.core.structure import Structure
+from pymatgen.core.composition import Composition
 from pymatgen.io.vaspio import Vasprun, Incar, Kpoints, Potcar, Poscar, \
     Outcar, Oszicar
 from pymatgen.io.cifio import CifWriter
 from pymatgen.symmetry.finder import SymmetryFinder
 from pymatgen.analysis.bond_valence import BVAnalyzer
 from pymatgen.util.io_utils import zopen
+from pymatgen.matproj.rest import MPRester
+from pymatgen.entries.computed_entries import ComputedEntry
 
 
 logger = logging.getLogger(__name__)
@@ -67,7 +70,8 @@ class VaspToDbTaskDrone(AbstractDrone):
     def __init__(self, host="127.0.0.1", port=27017, database="vasp",
                  user=None, password=None,  collection="tasks",
                  parse_dos=False, simulate_mode=False,
-                 additional_fields=None, update_duplicates=True):
+                 additional_fields=None, update_duplicates=True,
+                 mapi_key=None):
         """
         Args:
             host:
@@ -99,6 +103,19 @@ class VaspToDbTaskDrone(AbstractDrone):
             update_duplicates:
                 If True, if a duplicate path exists in the collection, the
                 entire doc is updated. Else, duplicates are skipped.
+            mapi_key:
+                A Materials API key. If this key is supplied,
+                the insertion code will attempt to use the Materials REST API
+                to calculate stability data for inserted calculations.
+                Stability assessment requires a large quantity of materials
+                data. E.g., to compute the stability of a new LixFeyOz
+                calculation, you need to the energies of all known
+                phases in the Li-Fe-O chemical system. Using
+                the Materials API, we can obtain the pre-calculated data from
+                the Materials Project.
+
+                Go to www.materialsproject.org/profile to generate or obtain
+                your API key.
         """
         self.host = host
         self.database = database
@@ -111,6 +128,7 @@ class VaspToDbTaskDrone(AbstractDrone):
         self.additional_fields = {} if not additional_fields \
             else additional_fields
         self.update_duplicates = update_duplicates
+        self.mapi_key = mapi_key
         if not simulate_mode:
             conn = MongoClient(self.host, self.port)
             db = conn[self.database]
@@ -131,6 +149,8 @@ class VaspToDbTaskDrone(AbstractDrone):
         try:
             d = self.get_task_doc(path, self.parse_dos,
                                   self.additional_fields)
+            if self.mapi_key is not None and d["state"] == "successful":
+                self.calculate_stability(d)
             tid = self._insert_doc(d)
             return tid
         except Exception as ex:
@@ -139,6 +159,19 @@ class VaspToDbTaskDrone(AbstractDrone):
             logger.error(traceback.format_exc(ex))
             return False
 
+    def calculate_stability(self, d):
+        m = MPRester(self.mapi_key)
+        functional = d["pseudo_potential"]["functional"]
+        syms = ["{} {}".format(functional, l)
+                for l in d["pseudo_potential"]["labels"]]
+        entry = ComputedEntry(Composition(d["unit_cell_formula"]),
+                              d["output"]["final_energy"],
+                              parameters={"hubbards": d["hubbards"],
+                                          "potcar_symbols": syms})
+        data = m.get_stability([entry])[0]
+        for k in ("e_above_hull", "decomposes_to"):
+            d["analysis"][k] = data[k]
+
     @classmethod
     def get_task_doc(cls, path, parse_dos=False, additional_fields=None):
         """
@@ -270,10 +303,15 @@ class VaspToDbTaskDrone(AbstractDrone):
         logger.info("Post-processing dir:{}".format(dir_name))
 
         fullpath = os.path.abspath(dir_name)
+
+        # VASP input generated by pymatgen's alchemy has a
+        # transformations.json file that keeps track of the origin of a
+        # particular structure. This is extremely useful for tracing back a
+        # result. If such a file is found, it is inserted into the task doc
+        # as d["transformations"]
         transformations = {}
         filenames = glob.glob(os.path.join(fullpath, "transformations.json*"))
         if len(filenames) >= 1:
-            # Handles the new style transformations file.
             with zopen(filenames[0], "rb") as f:
                 transformations = json.load(f)
                 try:
@@ -301,7 +339,18 @@ class VaspToDbTaskDrone(AbstractDrone):
 
         d["transformations"] = transformations
 
-        # Parse OUTCAR for additional information and run stats.
+        # Calculations done using custodian has a custodian.json,
+        # which tracks the jobs performed and any errors detected and fixed.
+        # This is useful for tracking what has actually be done to get a
+        # result. If such a file is found, it is inserted into the task doc
+        # as d["custodian"]
+        filenames = glob.glob(os.path.join(fullpath, "custodian.json*"))
+        if len(filenames) >= 1:
+            with zopen(filenames[0], "rb") as f:
+                d["custodian"] = json.load(f)
+
+        # Parse OUTCAR for additional information and run stats that are
+        # generally not in vasprun.xml.
         run_stats = {}
         for filename in glob.glob(os.path.join(fullpath, "OUTCAR*")):
             outcar = Outcar(filename)
@@ -556,10 +605,11 @@ def get_basic_analysis_and_error_checks(d):
     delta_vol = final_vol - initial_vol
     percent_delta_vol = delta_vol / initial_vol
     coord_num = get_coordination_numbers(d)
-    gap = d["calculations"][-1]["output"]["bandgap"]
-    cbm = d["calculations"][-1]["output"]["cbm"]
-    vbm = d["calculations"][-1]["output"]["vbm"]
-    is_direct = d["calculations"][-1]["output"]["is_gap_direct"]
+    calc = d["calculations"][-1]
+    gap = calc["output"]["bandgap"]
+    cbm = calc["output"]["cbm"]
+    vbm = calc["output"]["vbm"]
+    is_direct = calc["output"]["is_gap_direct"]
 
     if abs(percent_delta_vol) > 0.20:
         warning_msgs = ["Volume change > 20%"]
