@@ -24,7 +24,8 @@ DEFAULT_SETTINGS = [
     ("readonly_user", None),
     ("readonly_password", None),
     ("collection", "tasks"),
-    ("aliases_config", None)
+    ("aliases_config", None),
+    ("mapi_key", None)
 ]
 
 
@@ -38,6 +39,7 @@ def get_settings(config_file):
     else:
         return dict(DEFAULT_SETTINGS)
 
+
 class MongoJSONEncoder(json.JSONEncoder):
     def default(self, o):
         if isinstance(o, bson.objectid.ObjectId):
